<?php

namespace Drupal\timeline_block\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Entity\Node;
use Drupal\file\Entity\File;

/**
 * Provides the Timeline Block.
 *
 * @Block(
 *   id="timeline_block",
 *   admin_label = @Translation("Timeline Block"),
 * )
 */
class TimelineBlock extends BlockBase
{

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration()
  {
    return parent::defaultConfiguration() + [
      'timeline_block' => [],
    ];
  }

  /**
   * Overrides \Drupal\Core\Block\BlockBase::blockForm().
   *
   * Adds body and description fields to the block configuration form.
   */
  public function blockForm($form, FormStateInterface $form_state)
  {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    $items = [];
    if (isset($config['timeline_data'])) {
      $items = $config['timeline_data'];
      if ($items != '') {
        $items = json_decode($items);
      }
    }

    $form['timeline_header'] = array(
      '#type' => 'text_format',
      '#title' => t('Timeline header content'),
      '#description' => t('Timeline header content'),
      '#default_value' => $config['timeline_header'] ?? '',
      '#format' => $config['timeline_header_format'] ?? 'basic_html',
    );

    $form['#tree'] = TRUE;

    $form['items_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Timeline items'),
      '#prefix' => '<div id="items-fieldset-wrapper">',
      '#suffix' => '</div>',
    ];

    if (!$form_state->has('num_items')) {
      if (is_array($items)) {
        $count_items = count($items);
      } else {
        $count_items = 0;
      }
      $form_state->set('num_items', $count_items);
    }
    $number_of_items = $form_state->get('num_items');

    for ($i = 0; $i < $number_of_items; $i++) {

      $is_active_row = $form_state->get("row_" . $i);

      if ($is_active_row != 'inactive') {

        $j = $i + 1;
        $form['items_fieldset']['items'][$i] = [
          '#type' => 'details',
          '#title' => $this->t('Timeline ' . $j),
          '#prefix' => '<div id="items-fieldset-wrapper">',
          '#suffix' => '</div>',
          '#open' => TRUE,
        ];
        $form['items_fieldset']['items'][$i]['title'] = [
          '#type' => 'textfield',
          '#title' => t('Title'),
          '#description' => t('Timeline item title'),
          '#default_value' => $items[$i]->title ?? '',
        ];
        $form['items_fieldset']['items'][$i]['time'] = [
          '#type' => 'textfield',
          '#title' => t('Time'),
          '#description' => t('Timeline item time pase'),
          '#default_value' => $items[$i]->time ?? '',
        ];
        $form['items_fieldset']['items'][$i]['description'] = [
          '#type' => 'text_format',
          '#title' => t('Description'),
          '#description' => t('Timeline item description'),
          '#default_value' => $items[$i]->description ?? '',
          '#format' => $items[$i]->description_format ?? 'basic_html',
        ];
        $form['items_fieldset']['items'][$i]['weight'] = [
          '#type' => 'number',
          '#min' => 1,
          '#max' => 120,
          '#title' => t('Timeline item order weight'),
          '#description' => t('The weight field can be used to provide customized sorting of timeline item.'),
          '#default_value' => $items[$i]->weight ?? $j,
        ];
        $form['items_fieldset']['items'][$i]['remove_single_item'] = [
          '#type' => 'submit',
          '#value' => t('Remove item '),
          '#name' => 'row_' . $i,
          '#submit' => [[$this, 'removeItemCallback']],
          '#ajax' => [
            'callback' => [$this, 'addmoreCallback'],
            'wrapper' => 'items-fieldset-wrapper',
          ],
          '#button_type' => 'danger',
        ];
      }
    }

    $form['items_fieldset']['actions'] = [
      '#type' => 'actions',
    ];

    $form['items_fieldset']['actions']['add_item'] = [
      '#type' => 'submit',
      '#value' => t('Add timeline'),
      '#submit' => [[$this, 'addOne']],
      '#ajax' => [
        'callback' => [$this, 'addmoreCallback'],
        'wrapper' => 'items-fieldset-wrapper',
      ],
      '#button_type' => 'primary',
    ];

    $form['timeline_configuration'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Timeline configuration'),
      '#description' => $this->t('Make adjustments such as how many items will appear according to screen sizes and other configurations.'),
      '#prefix' => '<div id="items-fieldset-wrapper">',
      '#suffix' => '</div>',
    ];

    $form['timeline_configuration']['timeline_layout'] = [
      '#type' => 'select',
      '#title' => t('Timeline layout'),
      '#description' => t('Timeline layout'),
      '#options' => array(
        '1' => 'Layout 1',
        '2' => 'Layout 2',
        '3' => 'Layout 3',
        '4' => 'Layout 4',
        '5' => 'Layout 5',
        '6' => 'Layout 6',
        '7' => 'Layout 7',
        '8' => 'Layout 8',
        '9' => 'Layout 9',
        '10' => 'Layout 10',
      ),
      '#default_value' => $config['timeline_layout'] ?? '1',
    ];

    return $form;
  }

  /**
   * Submit handler for the "remove one" button.
   *
   * Decrements the max counter and causes a form rebuild.
   */
  public function removeItemCallback(array &$form, FormStateInterface $form_state)
  {
    $button_clicked = $form_state->getTriggeringElement()['#name'];

    $form_state->set($button_clicked, 'inactive');

    $form_state->setRebuild();

  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function addOne(array &$form, FormStateInterface $form_state)
  {
    $number_of_items = $form_state->get('num_items');
    $add_button = $number_of_items + 1;
    $form_state->set('num_items', $add_button);
    $form_state->setRebuild();
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @return mixed
   */
  public function addmoreCallback(array &$form, FormStateInterface $form_state)
  {
    // The form passed here is the entire form, not the subform that is
    // passed to non-AJAX callback.
    return $form['settings']['items_fieldset'];
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function removeCallback(array &$form, FormStateInterface $form_state)
  {
    $number_of_items = $form_state->get('num_items');
    if ($number_of_items > 1) {
      $remove_button = $number_of_items - 1;
      $form_state->set('num_items', $remove_button);
    }
    $form_state->setRebuild();
  }
  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function removeCallbackSingle(array &$form, FormStateInterface $form_state)
  {
    $number_of_items = $form_state->get('num_items');
    if ($number_of_items > 1) {
      $remove_button = $number_of_items - 1;
      $form_state->set('num_items', $remove_button);
    }
    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state)
  {
    $timeline_header = $this->configuration['timeline_header'] ?? '';
    $timeline_data_config = $this->configuration['timeline_data'] ?? '';
    $old_timeline_image_ids = get_timeline_description_images($timeline_data_config);
    $current_timeline_image_ids = array();
    $current_header_image_ids = array();
    $items = array();

    $values = $form_state->getValues();
    foreach ($values as $key => $value) {

      if ($key === 'items_fieldset') {
        if (isset($value['items']) && !empty($value['items'])) {
          $items = $value['items'];

          usort($items, function ($x, $y) {
            if (is_numeric($x['weight']) && is_numeric($y['weight'])) {
              return $x['weight'] - $y['weight'];
            }
          });

          foreach ($items as $key => $item) {
            if (trim($item['description']['value']) === '') {
              unset($items[$key]);
            } else {
              if (!is_numeric($item['weight'])) {
                $items[$key]['weight'] = 1;
              }

              $description_value = $items[$key]['description']['value'] ?? '';
              $description_format = $items[$key]['description']['format'] ?? 'basic_html';

              $items[$key]['description'] = $description_value;
              $items[$key]['description_format'] = $description_format;
              $new_img_ids = get_timeline_editor_files($description_value);

              $current_timeline_image_ids = array_merge($current_timeline_image_ids, $new_img_ids);
            }
          }

          $timeline_data = array_values($items);
          $timeline_data = json_encode($timeline_data);

          $this->configuration['timeline_data'] = $timeline_data;
        } else {
          $this->configuration['timeline_data'] = '';
        }
      }
    }

    if ($form_state->getValue('timeline_configuration')) {
      $timeline_configuration = $form_state->getValue('timeline_configuration');
      $this->configuration['timeline_layout'] = $timeline_configuration['timeline_layout'];
    }

    $timeline_header_content = $values['timeline_header']['value'] ?? '';
    $this->configuration['timeline_header'] = $timeline_header_content;
    $this->configuration['timeline_header_format'] = $values['timeline_header']['format'] ?? 'basic_html';

    $old_header_image_ids = get_timeline_editor_files($timeline_header);
    $current_header_image_ids = get_timeline_editor_files($timeline_header_content);

    $all_old_image_ids = array_merge($old_timeline_image_ids, $old_header_image_ids);
    $all_new_image_ids = array_merge($current_timeline_image_ids, $current_header_image_ids);

    // remove old images
    $result = array_diff($all_old_image_ids, $all_new_image_ids);

    if (!empty($result)) {
      foreach ($result as $key => $author_image_id) {
        $file = File::load($author_image_id);
        $file->setTemporary();
        $file->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build()
  {
    $config = $this->getConfiguration();
    $timeline_data_config = $this->configuration['timeline_data'] ?? '';
    $timeline_layout = $this->configuration['timeline_layout'] ?? 1;
    $timeline_header = $this->configuration['timeline_header'] ?? '';
    $timeline_settings = array(
      'timeline_layout' => $timeline_layout,
    );

    $timeline_data = array();
    if ($timeline_data_config != '') {
      $timeline_data = json_decode($timeline_data_config);
    }
    $build = [];
    $build['timeline'] = [
      '#theme' => 'timeline_block',
      '#timeline_data' => $timeline_data,
      '#timeline_settings' => $timeline_settings,
      '#timeline_header' => $timeline_header
    ];
    $build['#attached']['library'][] = 'timeline_block/timeline_block';
    return $build;
  }

}